@@ -1,11 +1,11 @@
 /***************************************
-  $Header: /home/amb/procmeter3/widgets/RCS/PMGraph.h 1.1 1998/09/19 15:29:06 amb Exp $
+  $Header: /home/amb/procmeter3/widgets/RCS/PMGraph.h 1.2 1999/02/13 11:37:24 amb Exp $
 
-  ProcMeter Graph Widget Public include file (for ProcMeter 3.0).
+  ProcMeter Graph Widget Public include file (for ProcMeter3 3.1).
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1996,98 Andrew M. Bishop
+  This file Copyright 1996,98,99 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -30,9 +30,11 @@ typedef struct _ProcMeterGraphRec      *ProcMeterGraphWidget;
 #define XtNgridUnits      "gridUnits"
 #define XtNgridForeground "gridForeground"
 #define XtNgridMin        "gridMin"
+#define XtNgridMax        "gridMax"
 
 #define XtCSolid         "Solid"
 #define XtCGridMin       "GridMin"
+#define XtCGridMax       "GridMax"
 
 /* Public functions */
 
