@@ -1,13 +1,13 @@
 /***************************************
-  $Header: /home/amb/procmeter3/RCS/xmenus.c 1.2 1998/09/26 09:44:59 amb Exp $
+  $Header: /home/amb/procmeter3/RCS/xmenus.c 1.13 1999/11/30 19:48:28 amb Exp $
 
-  ProcMeter - A system monitoring program for Linux.
+  ProcMeter - A system monitoring program for Linux - Version 3.2.
 
   X Window menus.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1998 Andrew M. Bishop
+  This file Copyright 1998,99 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -16,6 +16,7 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <unistd.h>
 
 #include <X11/X.h>
 #include <X11/Xlib.h>
@@ -30,34 +31,53 @@
 #include <X11/Xaw/Form.h>
 #include <X11/Xaw/Command.h>
 #include <X11/Xaw/AsciiText.h>
+#include <X11/cursorfont.h>
+
+#include "widgets/SubMenus.h"
 
 #include "xwindow.h"
 #include "procmeterp.h"
 
 
+/* This is already defined if using Xaw3d. */
+#ifndef XtNshadowWidth
+#define XtNshadowWidth "shadowWidth"
+#endif
+
+/*+ The orientation of the windows. +*/
+extern int vertical;
+
+/*+ The pane that contains all of the outputs. +*/
+extern Widget pane;
+
+/* Local functions */
+
 static void ModuleMenuStart(Widget w,XEvent *event,String *params,Cardinal *num_params);
-static void ModuleMenuEvent(Widget w,XEvent *event,String *params,Cardinal *num_params);
 static void OutputMenuStart(Widget w,XEvent *event,String *params,Cardinal *num_params);
-static void PropertiesMenuStart(Widget w,XEvent *event,String *params,Cardinal *num_params);
+static void FunctionsMenuStart(Widget w,XEvent *event,String *params,Cardinal *num_params);
 
-static void PopupAMenu(Widget menu,Widget w,XEvent *event);
-
-static void SelectModuleMenuCallback(Widget widget,XtPointer clientData,XtPointer callData);
 static void SelectOutputMenuCallback(Widget widget,XtPointer clientData,XtPointer callData);
-static void SelectPropertiesMenuCallback(Widget widget,XtPointer clientData,XtPointer callData);
+static void SelectFunctionsMenuCallback(Widget widget,XtPointer clientData,XtPointer callData);
+
 static void DonePropertiesDialogCallback(Widget widget,XtPointer clientData,XtPointer callData);
 static void AtomPropertiesDialogCloseCallback(Widget w,XtPointer va,XEvent* e,Boolean* vb);
 
+/* Menu widgets */
 
 static Widget module_menu;
-static Widget properties_menu,properties_dialog;
-static Widget prop_modname,prop_moddesc,prop_outname,prop_outdesc;
-static Boolean properties_popped_up=False;
+static Widget functions_menu,properties_dialog;
+static Widget func_run;
+static Widget prop_modname,prop_moddesc,
+              prop_outname,prop_outdesc,
+              prop_label,prop_type,prop_interval,prop_scale;
+static Boolean properties_popped_up=False,doing_move=False;
 
 XtActionsRec MenuActions[]={{"ModuleMenuStart",ModuleMenuStart},
-                            {"ModuleMenuEvent",ModuleMenuEvent},
                             {"OutputMenuStart",OutputMenuStart},
-                            {"PropertiesMenuStart",PropertiesMenuStart}};
+                            {"FunctionsMenuStart",FunctionsMenuStart}};
+
+/*+ The output that was used for the Functions menu. +*/
+static Output function_output;
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -68,166 +88,267 @@ XtActionsRec MenuActions[]={{"ModuleMenuStart",ModuleMenuStart},
 
 void CreateMenus(Widget parent)
 {
- Widget menulabel1,/*menulabel2,*/smeline1,/*smeline2,*/sme;
- Widget prop_form,prop_modlabel,prop_outlabel,prop_done;
+ Widget menulabel;
+ Widget menuline;
+ Widget func_prop,func_above,func_below,func_delete;
+ Widget prop_form,
+        prop_modlabel,prop_outlabel,
+        prop_lbllabel,prop_typlabel,prop_intlabel,prop_scllabel,
+        prop_done;
  Dimension width,height;
  char *string;
- XtPointer resource;
+ Arg args[3];
+ int nargs=0;
+ XtVarArgsList resources=NULL;
  Atom close_atom;
 
+ /* Add the application actions. */
+
  XtAppAddActions(app_context,MenuActions,sizeof(MenuActions)/sizeof(MenuActions[0]));
+ InitialiseSubMenus(app_context);
+
+ /* Sort out the resources in advance, this is required for the dialog box.
+    This is horrible because Xaw3d requires that you set the background when you
+    create the widget because the shadows don't update if you do it later. */
+
+ if((string=GetProcMeterRC("resources","menu-foreground")))
+   {XtSetArg(args[nargs],XtNforeground,StringToPixel(string));nargs++;}
+
+ if((string=GetProcMeterRC("resources","menu-background")))
+   {XtSetArg(args[nargs],XtNbackground,StringToPixel(string));nargs++;}
+
+ if((string=GetProcMeterRC("resources","menu-font")))
+   {XtSetArg(args[nargs],XtNfont,StringToFont(string));nargs++;}
+
+ if(nargs==1)
+    resources=XtVaCreateArgsList(NULL,
+                                 args[0].name,args[0].value,
+                                 NULL);
+ else if(nargs==2)
+    resources=XtVaCreateArgsList(NULL,
+                                 args[0].name,args[0].value,
+                                 args[1].name,args[1].value,
+                                 NULL);
+ else if(nargs==3)
+    resources=XtVaCreateArgsList(NULL,
+                                 args[0].name,args[0].value,
+                                 args[1].name,args[1].value,
+                                 args[2].name,args[2].value,
+                                 NULL);
+ else
+    resources=XtVaCreateArgsList(NULL,
+                                 NULL);
 
  /* The module menu */
 
  module_menu=XtVaCreatePopupShell("ModuleMenu",simpleMenuWidgetClass,parent,
                                   XtNlabel,"Modules",
+                                  XtVaNestedList,resources,
                                   NULL);
 
- XtOverrideTranslations(module_menu,XtParseTranslationTable("<BtnMotion>: highlight() ModuleMenuEvent()\n"
-                                                            "<BtnUp>:     MenuPopdown() notify() unhighlight() ModuleMenuEvent()\n"));
 
- menulabel1=XtNameToWidget(module_menu,"menuLabel");
+ menulabel=XtNameToWidget(module_menu,"menuLabel");
+ XtSetValues(menulabel,args,nargs);
 
- smeline1=XtVaCreateManagedWidget("line",smeLineObjectClass,module_menu,NULL);
+ menuline=XtVaCreateManagedWidget("line",smeLineObjectClass,module_menu,
+                                  XtVaNestedList,resources,
+                                  NULL);
 
- /* The properties menu */
+ /* The functions menu */
 
- properties_menu=XtVaCreatePopupShell("PropertiesMenu",simpleMenuWidgetClass,parent,
-                                      NULL);
+ functions_menu=XtVaCreatePopupShell("FunctionsMenu",simpleMenuWidgetClass,parent,
+                                     XtNlabel,"Functions",
+                                     XtVaNestedList,resources,
+                                     NULL);
 
-/*
- menulabel2=XtNameToWidget(properties_menu,"menuLabel");
-*/
+ menulabel=XtNameToWidget(functions_menu,"menuLabel");
+ XtSetValues(menulabel,args,nargs);
 
-/*
- smeline2=XtVaCreateManagedWidget("line",smeLineObjectClass,properties_menu,NULL);
-*/
+ menuline=XtVaCreateManagedWidget("line",smeLineObjectClass,functions_menu,
+                                  XtVaNestedList,resources,
+                                  NULL);
 
- sme=XtVaCreateManagedWidget("Properties",smeBSBObjectClass,properties_menu,
-                             XtNlabel,"Properties",
-                             XtNheight,10,
-                             NULL);
+ /* The functions menu items */
 
- XtAddCallback(sme,XtNcallback,SelectPropertiesMenuCallback,0);
+ func_prop=XtVaCreateManagedWidget("Properties",smeBSBObjectClass,functions_menu,
+                                   XtNlabel,"Properties",
+                                   XtNheight,10,
+                                   XtVaNestedList,resources,
+                                   NULL);
+
+ XtAddCallback(func_prop,XtNcallback,SelectFunctionsMenuCallback,0);
+
+ func_above=XtVaCreateManagedWidget("AboveOrLeft",smeBSBObjectClass,functions_menu,
+                                    XtNlabel,vertical?"Move To Above":"Move To Left Of",
+                                    XtNheight,10,
+                                    XtVaNestedList,resources,
+                                    NULL);
+
+ XtAddCallback(func_above,XtNcallback,SelectFunctionsMenuCallback,(XtPointer)1);
+
+ func_below=XtVaCreateManagedWidget("BelowOrRight",smeBSBObjectClass,functions_menu,
+                                    XtNlabel,vertical?"Move To Below":"Move To Right Of",
+                                    XtNheight,10,
+                                    XtVaNestedList,resources,
+                                    NULL);
+
+ XtAddCallback(func_below,XtNcallback,SelectFunctionsMenuCallback,(XtPointer)2);
+
+ func_delete=XtVaCreateManagedWidget("Delete",smeBSBObjectClass,functions_menu,
+                                     XtNlabel,"Delete",
+                                     XtNheight,10,
+                                     XtVaNestedList,resources,
+                                     NULL);
+
+ XtAddCallback(func_delete,XtNcallback,SelectFunctionsMenuCallback,(XtPointer)3);
+
+ func_run=XtVaCreateManagedWidget("Run",smeBSBObjectClass,functions_menu,
+                                  XtNlabel,"Run",
+                                  XtNheight,10,
+                                  XtVaNestedList,resources,
+                                  NULL);
+
+ XtAddCallback(func_run,XtNcallback,SelectFunctionsMenuCallback,(XtPointer)4);
 
  /* The properties_dialog */
 
  properties_dialog=XtVaCreatePopupShell("PropertiesDialog",topLevelShellWidgetClass,parent,
                                         XtNtitle,"ProcMeter Properties",XtNiconName,"ProcMeter Properties",
+                                        XtVaNestedList,resources,
                                         NULL);
 
  prop_form=XtVaCreateManagedWidget("PropertiesForm",formWidgetClass,properties_dialog,
-                                   XtNwidth,200,XtNheight,200,
+                                   XtNwidth,300,XtNheight,300,
+                                   XtVaNestedList,resources,
                                    NULL);
 
+ /* The properties dialog elements. */
+
  prop_modlabel=XtVaCreateManagedWidget("ModuleLabel",labelWidgetClass,prop_form,
                                        XtNlabel,"Module:",
-                                       XtNwidth,100,XtNresizable,True,
-                                       XtNleft,XawChainLeft,XtNright,XawRubber,XtNtop,XawChainTop,XtNbottom,XawChainTop,
-                                       XtNborderWidth,0,
+                                       XtNleft,XawChainLeft,XtNright,XawChainLeft,XtNtop,XawChainTop,XtNbottom,XawChainTop,
+                                       XtNborderWidth,0,XtNshadowWidth,0,
+                                       XtVaNestedList,resources,
                                        NULL);
+
  prop_modname=XtVaCreateManagedWidget("ModuleName",labelWidgetClass,prop_form,
                                       XtNlabel,"NNNNNNNNNNNNNNN",XtNjustify,XtJustifyLeft,
-                                      XtNwidth,100,XtNresizable,True,
-                                      XtNleft,XawRubber,XtNright,XawChainRight,XtNtop,XawChainTop,XtNbottom,XawChainTop,
+                                      XtNleft,XawChainLeft,XtNright,XawChainLeft,XtNtop,XawChainTop,XtNbottom,XawChainTop,
                                       XtNfromHoriz,prop_modlabel,
+                                      XtNborderWidth,0,XtNshadowWidth,0,
+                                      XtVaNestedList,resources,
                                       NULL);
 
  prop_moddesc=XtVaCreateManagedWidget("ModuleDesc",asciiTextWidgetClass,prop_form,
-                                      XtNheight,100,XtNwidth,200,XtNresizable,True,
+                                      XtNheight,80,XtNwidth,240,XtNresizable,True,
                                       XtNleft,XawChainLeft,XtNright,XawChainRight,XtNtop,XawChainTop,XtNbottom,XawChainTop,
                                       XtNfromVert,prop_modlabel,
                                       XtNtype,XawAsciiString,XtNstring,"NNNNNNNNNNNNNNN",
                                       XtNwrap,XawtextWrapWord,XtNeditType,XawtextRead,XtNdisplayCaret,False,
                                       XtNscrollVertical,XawtextScrollAlways,
+                                      XtVaNestedList,resources,
                                       NULL);
 
  prop_outlabel=XtVaCreateManagedWidget("OutputLabel",labelWidgetClass,prop_form,
                                        XtNlabel,"Output:",
                                        XtNleft,XawChainLeft,XtNright,XawChainLeft,XtNtop,XawChainTop,XtNbottom,XawChainTop,
                                        XtNfromVert,prop_moddesc,
-                                       XtNborderWidth,0,
+                                       XtNborderWidth,0,XtNshadowWidth,0,
+                                       XtVaNestedList,resources,
                                        NULL);
+
  prop_outname=XtVaCreateManagedWidget("OutputName",labelWidgetClass,prop_form,
                                       XtNlabel,"NNNNNNNNNNNNNNN",XtNjustify,XtJustifyLeft,
                                       XtNleft,XawChainLeft,XtNright,XawChainLeft,XtNtop,XawChainTop,XtNbottom,XawChainTop,
                                       XtNfromHoriz,prop_outlabel,XtNfromVert,prop_moddesc,
+                                      XtNborderWidth,0,XtNshadowWidth,0,
+                                      XtVaNestedList,resources,
                                       NULL);
 
  prop_outdesc=XtVaCreateManagedWidget("OutputDesc",asciiTextWidgetClass,prop_form,
-                                      XtNheight,100,XtNwidth,200,XtNresizable,True,
+                                      XtNheight,80,XtNwidth,240,XtNresizable,True,
                                       XtNleft,XawChainLeft,XtNright,XawChainRight,XtNtop,XawChainTop,XtNbottom,XawChainTop,
                                       XtNfromVert,prop_outlabel,
                                       XtNtype,XawAsciiString,XtNstring,"NNNNNNNNNNNNNNN",
                                       XtNwrap,XawtextWrapWord,XtNeditType,XawtextRead,XtNdisplayCaret,False,
                                       XtNscrollVertical,XawtextScrollAlways,
+                                      XtVaNestedList,resources,
                                       NULL);
 
- prop_done=XtVaCreateManagedWidget("Done",commandWidgetClass,prop_form,
-                                   XtNwidth,200,XtNresizable,False,
-                                   XtNleft,XawChainLeft,XtNright,XawChainRight,XtNtop,XawChainTop,XtNbottom,XawChainTop,
-                                   XtNfromVert,prop_outdesc,
+ prop_lbllabel=XtVaCreateManagedWidget("LabelLabel",labelWidgetClass,prop_form,
+                                       XtNlabel,"Label:",
+                                       XtNleft,XawChainLeft,XtNright,XawChainLeft,XtNtop,XawChainTop,XtNbottom,XawChainTop,
+                                       XtNfromVert,prop_outdesc,
+                                       XtNborderWidth,0,XtNshadowWidth,0,
+                                       XtVaNestedList,resources,
+                                       NULL);
+
+ prop_typlabel=XtVaCreateManagedWidget("TypeLabel",labelWidgetClass,prop_form,
+                                       XtNlabel,"Type:",
+                                       XtNleft,XawChainLeft,XtNright,XawChainLeft,XtNtop,XawChainTop,XtNbottom,XawChainTop,
+                                       XtNfromVert,prop_lbllabel,
+                                       XtNborderWidth,0,XtNshadowWidth,0,
+                                       XtVaNestedList,resources,
+                                       NULL);
+
+ prop_intlabel=XtVaCreateManagedWidget("IntervalLabel",labelWidgetClass,prop_form,
+                                       XtNlabel,"Interval:",
+                                       XtNleft,XawChainLeft,XtNright,XawChainLeft,XtNtop,XawChainTop,XtNbottom,XawChainTop,
+                                       XtNfromVert,prop_typlabel,
+                                       XtNborderWidth,0,XtNshadowWidth,0,
+                                       XtVaNestedList,resources,
+                                       NULL);
+
+ prop_scllabel=XtVaCreateManagedWidget("ScaleLabel",labelWidgetClass,prop_form,
+                                       XtNlabel,"Grid Spacing:",
+                                       XtNleft,XawChainLeft,XtNright,XawChainLeft,XtNtop,XawChainTop,XtNbottom,XawChainTop,
+                                       XtNfromVert,prop_intlabel,
+                                       XtNborderWidth,0,XtNshadowWidth,0,
+                                       XtVaNestedList,resources,
+                                       NULL);
+
+ prop_label=XtVaCreateManagedWidget("Label",labelWidgetClass,prop_form,
+                                    XtNlabel,"NNNNNNNNNNNNNNNN",XtNjustify,XtJustifyLeft,
+                                    XtNleft,XawChainLeft,XtNright,XawChainLeft,XtNtop,XawChainTop,XtNbottom,XawChainTop,
+                                    XtNfromHoriz,prop_scllabel,XtNfromVert,prop_outdesc,
+                                    XtNborderWidth,0,XtNshadowWidth,0,
+                                    XtVaNestedList,resources,
+                                    NULL);
+
+ prop_type=XtVaCreateManagedWidget("Type",labelWidgetClass,prop_form,
+                                   XtNlabel,"NNNNNNN",XtNjustify,XtJustifyLeft,
+                                   XtNleft,XawChainLeft,XtNright,XawChainLeft,XtNtop,XawChainTop,XtNbottom,XawChainTop,
+                                   XtNfromHoriz,prop_scllabel,XtNfromVert,prop_lbllabel,
+                                   XtNborderWidth,0,XtNshadowWidth,0,
+                                   XtVaNestedList,resources,
                                    NULL);
+
+ prop_interval=XtVaCreateManagedWidget("Interval",labelWidgetClass,prop_form,
+                                       XtNlabel,"NNNNNNN",XtNjustify,XtJustifyLeft,
+                                       XtNleft,XawChainLeft,XtNright,XawChainLeft,XtNtop,XawChainTop,XtNbottom,XawChainTop,
+                                       XtNfromHoriz,prop_scllabel,XtNfromVert,prop_typlabel,
+                                       XtNborderWidth,0,XtNshadowWidth,0,
+                                       XtVaNestedList,resources,
+                                       NULL);
+
+ prop_scale=XtVaCreateManagedWidget("Scale",labelWidgetClass,prop_form,
+                                    XtNlabel,"NNNNNNN",XtNjustify,XtJustifyLeft,
+                                    XtNleft,XawChainLeft,XtNright,XawChainLeft,XtNtop,XawChainTop,XtNbottom,XawChainTop,
+                                    XtNfromHoriz,prop_scllabel,XtNfromVert,prop_intlabel,
+                                    XtNborderWidth,0,XtNshadowWidth,0,
+                                    XtVaNestedList,resources,
+                                    NULL);
+
+ prop_done=XtVaCreateManagedWidget("Done",commandWidgetClass,prop_form,
+                                   XtNwidth,240,XtNresizable,True,
+                                   XtNleft,XawChainLeft,XtNright,XawChainRight,XtNtop,XawChainTop,XtNbottom,XawChainTop,
+                                   XtNfromVert,prop_scllabel,
+                                   XtVaNestedList,resources,
+                                   NULL);
+
  XtAddCallback(prop_done,XtNcallback,DonePropertiesDialogCallback,0);
 
- /* Add the resources. */
-
- if(((string=GetProcMeterRC("resources","menu-foreground"))) &&
-    (resource=(XtPointer)StringToPixel(string)))
-   {
-    XtVaSetValues(menulabel1,XtNforeground,*(Pixel*)resource,NULL);
-/*
-    XtVaSetValues(menulabel2,XtNforeground,*(Pixel*)resource,NULL);
-*/
-    XtVaSetValues(smeline1,XtNforeground,*(Pixel*)resource,NULL);
-/*
-    XtVaSetValues(smeline2,XtNforeground,*(Pixel*)resource,NULL);
-*/
-    XtVaSetValues(sme,XtNforeground,*(Pixel*)resource,NULL);
-    XtVaSetValues(properties_dialog,XtNforeground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_form,XtNforeground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_modlabel,XtNforeground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_modname,XtNforeground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_moddesc,XtNforeground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_outlabel,XtNforeground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_outname,XtNforeground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_outdesc,XtNforeground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_done,XtNforeground,*(Pixel*)resource,NULL);
-   }
-
- if(((string=GetProcMeterRC("resources","menu-background"))) &&
-    (resource=(XtPointer)StringToPixel(string)))
-   {
-    XtVaSetValues(module_menu,XtNbackground,*(Pixel*)resource,NULL);
-    XtVaSetValues(properties_menu,XtNbackground,*(Pixel*)resource,NULL);
-    XtVaSetValues(properties_dialog,XtNbackground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_form,XtNbackground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_modlabel,XtNbackground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_modname,XtNbackground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_moddesc,XtNbackground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_outlabel,XtNbackground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_outname,XtNbackground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_outdesc,XtNbackground,*(Pixel*)resource,NULL);
-    XtVaSetValues(prop_done,XtNbackground,*(Pixel*)resource,NULL);
-   }
-
- if(((string=GetProcMeterRC("resources","menu-font"))) &&
-    (resource=(XtPointer)StringToFont(string)))
-   {
-    XtVaSetValues(menulabel1,XtNfont,resource,NULL);
-/*
-    XtVaSetValues(menulabel2,XtNfont,resource,NULL);
-*/
-    XtVaSetValues(sme,XtNfont,resource,NULL);
-    XtVaSetValues(prop_form,XtNfont,resource,NULL);
-    XtVaSetValues(prop_modlabel,XtNfont,resource,NULL);
-    XtVaSetValues(prop_modname,XtNfont,resource,NULL);
-    XtVaSetValues(prop_moddesc,XtNfont,resource,NULL);
-    XtVaSetValues(prop_outlabel,XtNfont,resource,NULL);
-    XtVaSetValues(prop_outname,XtNfont,resource,NULL);
-    XtVaSetValues(prop_outdesc,XtNfont,resource,NULL);
-    XtVaSetValues(prop_done,XtNfont,resource,NULL);
-   }
+ XtFree(resources);
 
  /* Fixup the properties dialog box */
 
@@ -253,25 +374,44 @@ void CreateMenus(Widget parent)
 void AddModuleToMenu(Module module)
 {
  int i;
- Widget menulabel,smeline;
+ Widget menulabel,menuline,submenu=NULL,menuitem=NULL;
+ ProcMeterOutput *prevoutput=NULL;
  char *string;
- XtPointer resource;
  char menuname[32];
+ Arg args[3];
+ int nargs=0;
 
  if(!display)
     return;
 
+ /* Sort out the resources in advance. */
+
+ if(((string=GetProcMeterRC(module->module->name,"menu-foreground")) ||
+     (string=GetProcMeterRC("resources","menu-foreground"))))
+   {XtSetArg(args[nargs],XtNforeground,StringToPixel(string));nargs++;}
+
+ if(((string=GetProcMeterRC(module->module->name,"menu-background")) ||
+     (string=GetProcMeterRC("resources","menu-background"))))
+   {XtSetArg(args[nargs],XtNbackground,StringToPixel(string));nargs++;}
+
+ if(((string=GetProcMeterRC(module->module->name,"menu-font")) ||
+     (string=GetProcMeterRC("resources","menu-font"))))
+   {XtSetArg(args[nargs],XtNfont,StringToFont(string));nargs++;}
+
  /* Create a new menu. */
 
  sprintf(menuname,"%sMenu",module->module->name);
  module->submenu_widget=XtVaCreatePopupShell(menuname,simpleMenuWidgetClass,module_menu,
                                              XtNlabel,module->module->name,
                                              NULL);
+ XtSetValues(module->submenu_widget,args,nargs);
 
  menulabel=XtNameToWidget(module->submenu_widget,"menuLabel");
+ XtSetValues(menulabel,args,nargs);
 
- smeline=XtVaCreateManagedWidget("line",smeLineObjectClass,module->submenu_widget,
-                                 NULL);
+ menuline=XtVaCreateManagedWidget("line",smeLineObjectClass,module->submenu_widget,
+                                  NULL);
+ XtSetValues(menuline,args,nargs);
 
  /* Add an entry to the module menu */
 
@@ -279,32 +419,9 @@ void AddModuleToMenu(Module module)
                                                   XtNlabel,module->module->name,
                                                   XtNheight,10,
                                                   NULL);
+ XtSetValues(module->menu_item_widget,args,nargs);
 
- XtAddCallback(module->menu_item_widget,XtNcallback,SelectModuleMenuCallback,(XtPointer)module->module);
-
- /* Add the resources. */
-
- if(((string=GetProcMeterRC(module->module->name,"menu-foreground")) ||
-     (string=GetProcMeterRC("resources","menu-foreground"))) &&
-    (resource=(XtPointer)StringToPixel(string)))
-   {
-    XtVaSetValues(menulabel,XtNforeground,*(Pixel*)resource,NULL);
-    XtVaSetValues(smeline,XtNforeground,*(Pixel*)resource,NULL);
-    XtVaSetValues(module->menu_item_widget,XtNforeground,*(Pixel*)resource,NULL);
-   }
-
- if(((string=GetProcMeterRC(module->module->name,"menu-background")) ||
-     (string=GetProcMeterRC("resources","menu-background"))) &&
-    (resource=(XtPointer)StringToPixel(string)))
-    XtVaSetValues(module->submenu_widget,XtNbackground,*(Pixel*)resource,NULL);
-
- if(((string=GetProcMeterRC(module->module->name,"menu-font")) ||
-     (string=GetProcMeterRC("resources","menu-font"))) &&
-    (resource=(XtPointer)StringToFont(string)))
-   {
-    XtVaSetValues(menulabel,XtNfont,resource,NULL);
-    XtVaSetValues(module->menu_item_widget,XtNfont,resource,NULL);
-   }
+ AddSubMenu(module->menu_item_widget,module->submenu_widget);
 
  /* Add entries to it for each output. */
 
@@ -313,14 +430,42 @@ void AddModuleToMenu(Module module)
     Widget sme;
     Pixmap bitmap=CircleBitmap;
 
+    if(module->outputs[i]->output!=prevoutput)
+      {
+       menuitem=XtVaCreateManagedWidget(module->outputs[i]->output->name,smeBSBObjectClass,module->submenu_widget,
+                                       XtNlabel,module->outputs[i]->label,
+                                       XtNheight,10,
+                                       NULL);
+
+       if(((string=GetProcMeterRC2(module->module->name,module->outputs[i]->output->name,"menu-foreground")) ||
+           (string=GetProcMeterRC(module->module->name,"menu-foreground")) ||
+           (string=GetProcMeterRC("resources","menu-foreground"))))
+          XtVaSetValues(menuitem,XtNforeground,StringToPixel(string),NULL);
+
+       if(((string=GetProcMeterRC2(module->module->name,module->outputs[i]->output->name,"menu-font")) ||
+           (string=GetProcMeterRC(module->module->name,"menu-font")) ||
+           (string=GetProcMeterRC("resources","menu-font"))))
+          XtVaSetValues(menuitem,XtNfont,StringToFont(string),NULL);
+
+       submenu=XtVaCreatePopupShell("Type",simpleMenuWidgetClass,module->submenu_widget,
+                                    NULL);
+       XtSetValues(submenu,args,nargs);
+
+       AddSubMenu(menuitem,submenu);
+
+       prevoutput=module->outputs[i]->output;
+      }
+
     if(module->outputs[i]->type==PROCMETER_GRAPH)
        bitmap=GraphBitmap;
     else if(module->outputs[i]->type==PROCMETER_TEXT)
        bitmap=TextBitmap;
+    else if(module->outputs[i]->type==PROCMETER_BAR)
+       bitmap=BarBitmap;
 
-    sme=XtVaCreateManagedWidget(module->outputs[i]->output->name,smeBSBObjectClass,module->submenu_widget,
-                                XtNlabel,module->outputs[i]->output->name,
-                                XtNleftMargin,12,XtNrightMargin,20,
+    sme=XtVaCreateManagedWidget(module->outputs[i]->output->name,smeBSBObjectClass,submenu,
+                                XtNlabel,"",
+                                XtNleftMargin,10,XtNrightMargin,20,
                                 XtNrightBitmap,bitmap,
                                 XtNheight,10,
                                 NULL);
@@ -329,21 +474,8 @@ void AddModuleToMenu(Module module)
 
     if(((string=GetProcMeterRC2(module->module->name,module->outputs[i]->output->name,"menu-foreground")) ||
         (string=GetProcMeterRC(module->module->name,"menu-foreground")) ||
-        (string=GetProcMeterRC("resources","menu-foreground"))) &&
-       (resource=(XtPointer)StringToPixel(string)))
-       XtVaSetValues(sme,XtNforeground,*(Pixel*)resource,NULL);
-
-    if(((string=GetProcMeterRC2(module->module->name,module->outputs[i]->output->name,"menu-font")) ||
-        (string=GetProcMeterRC(module->module->name,"menu-background")) ||
-        (string=GetProcMeterRC("resources","menu-background"))) &&
-       (resource=(XtPointer)StringToPixel(string)))
-       XtVaSetValues(sme,XtNbackground,*(Pixel*)resource,NULL);
-
-    if(((string=GetProcMeterRC2(module->module->name,module->outputs[i]->output->name,"menu-font")) ||
-        (string=GetProcMeterRC(module->module->name,"menu-font")) ||
-        (string=GetProcMeterRC("resources","menu-font"))) &&
-       (resource=(XtPointer)StringToFont(string)))
-       XtVaSetValues(sme,XtNfont,resource,NULL);
+        (string=GetProcMeterRC("resources","menu-foreground"))))
+       XtVaSetValues(sme,XtNforeground,StringToPixel(string),NULL);
 
     module->outputs[i]->menu_item_widget=sme;
     module->outputs[i]->output_widget=NULL;
@@ -377,7 +509,7 @@ void AddMenuToOutput(Widget widget,Module module,Output output)
  if(output)
    {
     char string[80];
-    sprintf(string,"<Btn1Down>: PropertiesMenuStart(%s.%s)",module->module->name,output->output->name);
+    sprintf(string,"<Btn1Down>: FunctionsMenuStart(%s)",module->module->name);
     XtOverrideTranslations(widget,XtParseTranslationTable(string));
    }
 }
@@ -410,23 +542,6 @@ void DestroyMenus(void)
 }
 
 
-/*++++++++++++++++++++++++++++++++++++++
-  The callback that is called by the module being selected on the module menu.
-
-  Widget widget The widget that the callback came from.
-
-  XtPointer clientData The client data from the callback.
-
-  XtPointer callData The call data from the callback.
-
-  This function is only ever called from the Xt Intrinsics routines.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static void SelectModuleMenuCallback(Widget widget,XtPointer clientData,XtPointer callData)
-{
-}
-
-
 /*++++++++++++++++++++++++++++++++++++++
   The callback that is called by the output being selected on the output menu.
 
@@ -448,7 +563,7 @@ static void SelectOutputMenuCallback(Widget widget,XtPointer clientData,XtPointe
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  The callback that is called by something being selected on the properties menu.
+  The callback that is called by an item being selected on the functions menu.
 
   Widget widget The widget that the callback came from.
 
@@ -459,9 +574,9 @@ static void SelectOutputMenuCallback(Widget widget,XtPointer clientData,XtPointe
   This function is only ever called from the Xt Intrinsics routines.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void SelectPropertiesMenuCallback(Widget widget,XtPointer clientData,XtPointer callData)
+static void SelectFunctionsMenuCallback(Widget widget,XtPointer clientData,XtPointer callData)
 {
- if((int)clientData==0)
+ if((int)clientData==0)         /* Properties */
    {
     Window root_return, child_return;
     int root_x_return, root_y_return;
@@ -478,6 +593,21 @@ static void SelectPropertiesMenuCallback(Widget widget,XtPointer clientData,XtPo
     XtPopup(properties_dialog,XtGrabNone);
     properties_popped_up=True;
    }
+ else if((int)clientData==1 || (int)clientData==2) /* Above / Below */
+   {
+    doing_move=(int)clientData;
+
+    XtGrabPointer(pane,True,ButtonPressMask|ButtonReleaseMask,GrabModeAsync,GrabModeAsync,None,XCreateFontCursor(XtDisplay(pane),XC_hand1),CurrentTime);
+   }
+ else if((int)clientData==3)    /* Delete */
+   {
+    AddRemoveOutput(function_output);
+   }
+ else if((int)clientData==4)    /* Run */
+   {
+    if(function_output->menu_run.flag)
+       RunProgram(&function_output->menu_run);
+   }
 }
 
 
@@ -537,72 +667,7 @@ static void AtomPropertiesDialogCloseCallback(Widget w,XtPointer va,XEvent* e,Bo
 
 static void ModuleMenuStart(Widget w,XEvent *event,String *params,Cardinal *num_params)
 {
- PopupAMenu(module_menu,w,event);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Do something with the sub-menus when there is an interesting button event.
-
-  Widget w The widget that caused the event.
-
-  XEvent *event The event that caused the callback.
-
-  String *params The parameters from the callback.
-
-  Cardinal *num_params The number of parameters.
-
-  This function is only ever called from the Xt Intrinsics routines.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static void ModuleMenuEvent(Widget w,XEvent *event,String *params,Cardinal *num_params)
-{
- static Widget lastitem=NULL,lastmenu=NULL;
- Widget sme;
- Module *module;
-
- sme=XawSimpleMenuGetActiveEntry(w);
-
- if((!sme || sme!=lastitem) && lastmenu)
-   {
-    XtPopdown(lastmenu);
-    lastmenu=lastitem=NULL;
-   }
-
- if(!sme || sme==lastitem)
-    return;
-
- lastitem=sme;
-
- for(module=Modules;*module;module++)
-    if(sme==(*module)->menu_item_widget)
-      {
-       Position root_x,root_y;
-       Dimension width,height;
-
-       lastmenu=(*module)->submenu_widget;
-
-       XtTranslateCoords(w,0,event->xbutton.y,&root_x,&root_y);
-       XtVaGetValues(lastmenu,XtNheight,&height,NULL);
-
-       root_y-=5;
-       if((DisplayHeight(display,DefaultScreen(display))-height)<root_y)
-          root_y=DisplayHeight(display,DefaultScreen(display))-height;
-       if(root_y<0)
-          root_y=0;
-
-       if(root_x<(DisplayWidth(display,DefaultScreen(display))/2))
-         {
-          XtVaGetValues(w,XtNwidth,&width,NULL);
-          XtVaSetValues(lastmenu,XtNx,root_x+width,XtNy,root_y,NULL);
-         }
-       else
-         {
-          XtVaGetValues(lastmenu,XtNwidth,&width,NULL);
-          XtVaSetValues(lastmenu,XtNx,root_x-width,XtNy,root_y,NULL);
-         }
-       XtPopup(lastmenu,XtGrabNone);
-      }
+ PopupMenuHere(module_menu,w,event);
 }
 
 
@@ -624,7 +689,7 @@ static void OutputMenuStart(Widget w,XEvent *event,String *params,Cardinal *num_
 {
  Widget menu=XtNameToWidget(module_menu,params[0]);
 
- PopupAMenu(menu,w,event);
+ PopupMenuHere(menu,w,event);
 }
 
 
@@ -642,23 +707,56 @@ static void OutputMenuStart(Widget w,XEvent *event,String *params,Cardinal *num_
   This function is only ever called from the Xt Intrinsics routines.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void PropertiesMenuStart(Widget w,XEvent *event,String *params,Cardinal *num_params)
+static void FunctionsMenuStart(Widget w,XEvent *event,String *params,Cardinal *num_params)
 {
- Output *outputp=NULL;
- Module *modulep=NULL;
+ static Output *outputp=NULL;
+ static Module *modulep=NULL;
+ char string[24];
 
  for(modulep=Modules;*modulep;modulep++)
-    if(!strncmp((*modulep)->module->name,params[0],strlen((*modulep)->module->name)) &&
-       params[0][strlen((*modulep)->module->name)]=='.')
+    if(!strcmp((*modulep)->module->name,params[0]))
       {
        for(outputp=(*modulep)->outputs;*outputp;outputp++)
-          if(!strcmp((*outputp)->output->name,&params[0][strlen((*modulep)->module->name)+1]))
+          if(w==(*outputp)->output_widget)
              break;
        break;
       }
 
- if(!*modulep)
+ if(doing_move)
+   {
+    XtUngrabPointer(pane,CurrentTime);
+
+    MoveOutput(function_output,*outputp,doing_move);
+    doing_move=0;
     return;
+   }
+
+ if(!*modulep || !*outputp)
+    return;
+
+ function_output=*outputp;
+
+ if((*outputp)->menu_run.flag)
+   {
+    char *r=(*outputp)->menu_run.command;
+
+    strncpy(string,"Run '",16);
+    strncpy(string+5,r,8);
+    if(strlen(r)>8)
+       strcat(string," ...'");
+    else
+       strcat(string,"'");
+ 
+    XtVaSetValues(func_run,XtNlabel,string,NULL);
+    XtSetSensitive(func_run,True);
+   }
+ else
+   {
+    XtVaSetValues(func_run,XtNlabel,"Run",NULL);
+    XtSetSensitive(func_run,False);
+   }
+
+ /* Set up the properties window. */
 
  XtVaSetValues(prop_modname,XtNlabel,(*modulep)->module->name,NULL);
  XtVaSetValues(prop_moddesc,XtNstring,(*modulep)->module->description,NULL);
@@ -666,38 +764,40 @@ static void PropertiesMenuStart(Widget w,XEvent *event,String *params,Cardinal *
  XtVaSetValues(prop_outname,XtNlabel,(*outputp)->output->name,NULL);
  XtVaSetValues(prop_outdesc,XtNstring,(*outputp)->output->description,NULL);
 
+ XtVaSetValues(prop_label,XtNlabel,(*outputp)->label,NULL);
+
+ if((*outputp)->type==PROCMETER_GRAPH)
+    XtVaSetValues(prop_type,XtNlabel,"Graph",NULL);
+ else if((*outputp)->type==PROCMETER_TEXT)
+    XtVaSetValues(prop_type,XtNlabel,"Text",NULL);
+ else if((*outputp)->type==PROCMETER_BAR)
+    XtVaSetValues(prop_type,XtNlabel,"Bar",NULL);
+
+ if((*outputp)->output->interval)
+    sprintf(string,"%d s",(*outputp)->output->interval);
+ else
+    strcpy(string,"Never");
+ XtVaSetValues(prop_interval,XtNlabel,string,NULL);
+
+ if((*outputp)->type&(PROCMETER_GRAPH|PROCMETER_BAR))
+   {
+    char str[16];
+    sprintf(str,(*outputp)->output->graph_units,(*outputp)->output->graph_scale);
+    if(*str=='(')
+       strcpy(string,str+1);
+    else
+       strcpy(string,str);
+    if(string[strlen(string)-1]==')')
+       string[strlen(string)-1]=0;
+    XtVaSetValues(prop_scale,XtNlabel,string,NULL);
+    XtSetSensitive(prop_scale,True);
+   }
+ else
+   {
+    XtVaSetValues(prop_scale,XtNlabel,"n/a",NULL);
+    XtSetSensitive(prop_scale,False);
+   }
+
  if(!properties_popped_up)
-    PopupAMenu(properties_menu,w,event);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Popup a menu, a replacement for the main part of the XawPositionSimpleMenu() and MenuPopup() translations.
-
-  Widget menu The menu to popup.
-
-  Widget w The widget causing the event.
-
-  XEvent *event The event that causes the popup.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static void PopupAMenu(Widget menu,Widget w,XEvent *event)
-{
- Position root_x,root_y;
- Dimension width,height;
-
- XtRealizeWidget(menu);
-
- XtVaGetValues(menu,XtNwidth,&width,XtNheight,&height,NULL);
- XtTranslateCoords(w,event->xbutton.x,event->xbutton.y,&root_x,&root_y);
-
- root_y-=5;
- if((DisplayHeight(display,DefaultScreen(display))-height)<root_y)
-    root_y=DisplayHeight(display,DefaultScreen(display))-height;
- if(root_y<0)
-    root_y=0;
-
- XtVaSetValues(menu,XtNx,root_x-width/2,XtNy,root_y,NULL);
- XtPopupSpringLoaded(menu);
- XtGrabPointer(menu,True,ButtonReleaseMask|ButtonPressMask,GrabModeAsync,GrabModeAsync,None,None,CurrentTime);
+    PopupMenuHere(functions_menu,w,event);
 }
