@@ -1,13 +1,13 @@
 /***************************************
-  $Header: /home/amb/procmeter3/RCS/xwindow.c 1.3 1998/10/24 09:02:51 amb Exp $
+  $Header: /home/amb/procmeter3/RCS/xwindow.c 1.10 1999/10/05 17:54:10 amb Exp $
 
-  ProcMeter - A system monitoring program for Linux (v3.0a).
+  ProcMeter - A system monitoring program for Linux - Version 3.2.
 
   X Windows interface.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1997,98 Andrew M. Bishop
+  This file Copyright 1997,98,99 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -30,11 +30,13 @@
 #include "widgets/PMGeneric.h"
 #include "widgets/PMGraph.h"
 #include "widgets/PMText.h"
+#include "widgets/PMBar.h"
 
 #include "procmeter.h"
 #include "procmeterp.h"
 #include "xwindow.h"
 
+
 #define MINHEIGHT 30
 #define MINWIDTH  60
 
@@ -60,7 +62,7 @@ extern int quit;
 static int sleeping;
 
 /*+ The pane that contains all of the outputs. +*/
-static Widget pane;
+Widget pane;
 
 /*+ A list of the outputs that are currently visible. +*/
 static Output *displayed=NULL;
@@ -86,7 +88,7 @@ void StartX(int *argc,char **argv)
  char *string;
 
  if((string=GetProcMeterRC("resources","horizontal")) &&
-    *StringToBoolean(string))
+    StringToBoolean(string))
     vertical=0;
 
  /* Initialise the display */
@@ -116,6 +118,9 @@ void StartX(int *argc,char **argv)
                               XtNorientation,vertical?XtorientVertical:XtorientHorizontal,
                               NULL);
 
+ if((string=GetProcMeterRC("resources","background")))
+    XtVaSetValues(pane,XtNbackground,StringToPixel(string),NULL);
+
  XtAddEventHandler(pane,StructureNotifyMask,False,(XtEventHandler)ResizePaneCallback,NULL);
 
  AddMenuToOutput(pane,NULL,NULL);
@@ -224,6 +229,15 @@ void UpdateX(time_t now)
                }
              else if((*output)->type==PROCMETER_TEXT)
                 ProcMeterTextWidgetChangeData((*output)->output_widget,(*output)->output->text_value);
+             else if((*output)->type==PROCMETER_BAR)
+               {
+                long value=(*output)->output->graph_value;
+                if(value<0)
+                   value=0;
+                if(value>65535)
+                   value=65535;
+                ProcMeterBarWidgetAddDatum((*output)->output_widget,value);
+               }
             }
 
           last=(*output)->output;
@@ -233,6 +247,10 @@ void UpdateX(time_t now)
 
 /*++++++++++++++++++++++++++++++++++++++
   Add the default outputs at startup.
+
+  int argc The number of command line arguments.
+
+  char **argv The command line arguments.
   ++++++++++++++++++++++++++++++++++++++*/
 
 void AddDefaultOutputs(int argc,char **argv)
@@ -242,6 +260,57 @@ void AddDefaultOutputs(int argc,char **argv)
  char *string;
  int arg;
 
+ if((string=GetProcMeterRC("startup","order")))
+   {
+    char *s=string;
+
+    while(*s && *s==' ')
+       s++;
+
+    while(*s)
+      {
+       int found=0;
+
+       for(modulep=Modules;*modulep;modulep++)
+          if(!strncmp((*modulep)->module->name,s,strlen((*modulep)->module->name)) &&
+             s[strlen((*modulep)->module->name)]=='.')
+            {
+             for(outputp=(*modulep)->outputs;*outputp;outputp++)
+                if(!strncmp((*outputp)->output->name,&s[strlen((*modulep)->module->name)+1],strlen((*outputp)->output->name)) &&
+                   (s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]=='-' ||
+                    s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]==' ' ||
+                    s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]==0))
+                  {
+                   if((s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]==' ' ||
+                       s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]==0) &&
+                      !(*outputp)->output_widget)
+                      AddRemoveOutput(*outputp);
+                   else if(s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+2]=='g' &&
+                           (*outputp)->type==PROCMETER_GRAPH &&
+                           !(*outputp)->output_widget)
+                      AddRemoveOutput(*outputp);
+                   else if(s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+2]=='t' &&
+                           (*outputp)->type==PROCMETER_TEXT &&
+                           !(*outputp)->output_widget)
+                      AddRemoveOutput(*outputp);
+                   else if(s[strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+2]=='b' &&
+                           (*outputp)->type==PROCMETER_BAR &&
+                           !(*outputp)->output_widget)
+                      AddRemoveOutput(*outputp);
+                   found=1;
+                  }
+
+             if(found)
+                break;
+            }
+
+       while(*s && *s!=' ')
+          s++;
+       while(*s && *s==' ')
+          s++;
+      }
+   }
+
  for(arg=1;arg<argc;arg++)
    {
     int found=0;
@@ -255,13 +324,20 @@ void AddDefaultOutputs(int argc,char **argv)
                 (argv[arg][strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]=='-' ||
                  argv[arg][strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1]==0))
                {
-                if(!argv[arg][strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1])
+                if(!argv[arg][strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+1] &&
+                   !(*outputp)->output_widget)
                    AddRemoveOutput(*outputp);
                 else if(argv[arg][strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+2]=='g' &&
-                        (*outputp)->type==PROCMETER_GRAPH)
+                        (*outputp)->type==PROCMETER_GRAPH &&
+                        !(*outputp)->output_widget)
                    AddRemoveOutput(*outputp);
                 else if(argv[arg][strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+2]=='t' &&
-                        (*outputp)->type==PROCMETER_TEXT)
+                        (*outputp)->type==PROCMETER_TEXT &&
+                        !(*outputp)->output_widget)
+                   AddRemoveOutput(*outputp);
+                else if(argv[arg][strlen((*modulep)->module->name)+strlen((*outputp)->output->name)+2]=='b' &&
+                        (*outputp)->type==PROCMETER_BAR &&
+                        !(*outputp)->output_widget)
                    AddRemoveOutput(*outputp);
                 found=1;
                }
@@ -274,22 +350,12 @@ void AddDefaultOutputs(int argc,char **argv)
        fprintf(stderr,"ProcMeter: Unrecognised output '%s'\n",argv[arg]);
    }
 
- for(modulep=Modules;*modulep;modulep++)
-    for(outputp=(*modulep)->outputs;*outputp;outputp++)
-       if(!(*outputp)->output_widget)
-          if(((*outputp)->type==PROCMETER_GRAPH &&
-              (string=GetProcMeterRC2((*modulep)->module->name,(*outputp)->output->name,"startup-graph")) &&
-              *StringToBoolean(string)) ||
-             ((*outputp)->type==PROCMETER_TEXT &&
-              (string=GetProcMeterRC2((*modulep)->module->name,(*outputp)->output->name,"startup-text")) &&
-              *StringToBoolean(string)))
-             AddRemoveOutput(*outputp);
-
  initialising=0;
 
  ResizePane();
 }
 
+
 /*++++++++++++++++++++++++++++++++++++++
   Add or remove an output
 
@@ -319,9 +385,8 @@ void AddRemoveOutput(Output output)
    }
  else
    {
-    Widget w;
-    char *string;
-    XtPointer resource;
+    Widget w=NULL;
+    char *string,str[16];
     Arg args[16];
     int nargs=0;
     Output *outputp=NULL;
@@ -343,64 +408,62 @@ void AddRemoveOutput(Output output)
 
     if(((string=GetProcMeterRC2(module->module->name,output->output->name,"foreground")) ||
         (string=GetProcMeterRC(module->module->name,"foreground")) ||
-        (string=GetProcMeterRC("resources","foreground"))) &&
-       (resource=(XtPointer)StringToPixel(string)))
-      {XtSetArg(args[nargs],XtNforeground,*(Pixel*)resource);nargs++;}
+        (string=GetProcMeterRC("resources","foreground"))))
+      {XtSetArg(args[nargs],XtNforeground,StringToPixel(string));nargs++;}
 
     if(((string=GetProcMeterRC2(module->module->name,output->output->name,"background")) ||
         (string=GetProcMeterRC(module->module->name,"background")) ||
-        (string=GetProcMeterRC("resources","background"))) &&
-       (resource=(XtPointer)StringToPixel(string)))
-      {XtSetArg(args[nargs],XtNbackground,*(Pixel*)resource);nargs++;}
+        (string=GetProcMeterRC("resources","background"))))
+      {XtSetArg(args[nargs],XtNbackground,StringToPixel(string));nargs++;}
 
     if(((string=GetProcMeterRC2(module->module->name,output->output->name,"label-font")) ||
         (string=GetProcMeterRC(module->module->name,"label-font")) ||
-        (string=GetProcMeterRC("resources","label-font"))) &&
-       (resource=StringToFont(string)))
-      {XtSetArg(args[nargs],XtNlabelFont,resource);nargs++;}
+        (string=GetProcMeterRC("resources","label-font"))))
+      {XtSetArg(args[nargs],XtNlabelFont,StringToFont(string));nargs++;}
 
     if(((string=GetProcMeterRC2(module->module->name,output->output->name,"label-foreground")) ||
         (string=GetProcMeterRC(module->module->name,"label-foreground")) ||
         (string=GetProcMeterRC("resources","label-foreground")) ||
         (string=GetProcMeterRC2(module->module->name,output->output->name,"foreground")) ||
         (string=GetProcMeterRC(module->module->name,"foreground")) ||
-        (string=GetProcMeterRC("resources","foreground"))) &&
-       (resource=(XtPointer)StringToPixel(string)))
-      {XtSetArg(args[nargs],XtNlabelForeground,*(Pixel*)resource);nargs++;}
+        (string=GetProcMeterRC("resources","foreground"))))
+      {XtSetArg(args[nargs],XtNlabelForeground,StringToPixel(string));nargs++;}
 
     if(((string=GetProcMeterRC2(module->module->name,output->output->name,"label-position")) ||
         (string=GetProcMeterRC(module->module->name,"label-position")) ||
-        (string=GetProcMeterRC("resources","label-position"))) &&
-       (resource=(XtPointer)StringToLabelPosition(string)))
-      {XtSetArg(args[nargs],XtNlabelPosition,*(int*)resource);nargs++;}
+        (string=GetProcMeterRC("resources","label-position"))))
+      {XtSetArg(args[nargs],XtNlabelPosition,StringToLabelPosition(string));nargs++;}
 
     if(output->type==PROCMETER_GRAPH)
       {
        if(((string=GetProcMeterRC2(module->module->name,output->output->name,"grid-foreground")) ||
            (string=GetProcMeterRC(module->module->name,"grid-foreground")) ||
-           (string=GetProcMeterRC("resources","grid-foreground"))) &&
-          (resource=(XtPointer)StringToPixel(string)))
-         {XtSetArg(args[nargs],XtNgridForeground,*(Pixel*)resource);nargs++;}
+           (string=GetProcMeterRC("resources","grid-foreground"))))
+         {XtSetArg(args[nargs],XtNgridForeground,StringToPixel(string));nargs++;}
 
        if(((string=GetProcMeterRC2(module->module->name,output->output->name,"graph-solid")) ||
            (string=GetProcMeterRC(module->module->name,"graph-solid")) ||
-           (string=GetProcMeterRC("resources","graph-solid"))) &&
-          (resource=(XtPointer)StringToBoolean(string)))
-         {XtSetArg(args[nargs],XtNsolid,*(Boolean*)resource);nargs++;}
+           (string=GetProcMeterRC("resources","graph-solid"))))
+         {XtSetArg(args[nargs],XtNsolid,StringToBoolean(string));nargs++;}
 
        if(((string=GetProcMeterRC2(module->module->name,output->output->name,"grid-min")) ||
            (string=GetProcMeterRC(module->module->name,"grid-min")) ||
-           (string=GetProcMeterRC("resources","grid-min"))) &&
-          (resource=(XtPointer)StringToInt(string)))
-         {XtSetArg(args[nargs],XtNgridMin,*(int*)resource);nargs++;}
+           (string=GetProcMeterRC("resources","grid-min"))))
+         {XtSetArg(args[nargs],XtNgridMin,StringToInt(string));nargs++;}
+
+       if(((string=GetProcMeterRC2(module->module->name,output->output->name,"grid-max")) ||
+           (string=GetProcMeterRC(module->module->name,"grid-max")) ||
+           (string=GetProcMeterRC("resources","grid-max"))))
+         {XtSetArg(args[nargs],XtNgridMax,StringToInt(string));nargs++;}
 
        if(vertical)
          {XtSetArg(args[nargs],XtNmin,MINHEIGHT);nargs++;}
        else
          {XtSetArg(args[nargs],XtNmin,MINWIDTH);nargs++;}
 
-       XtSetArg(args[nargs],XtNlabel,output->output->name);nargs++;
-       XtSetArg(args[nargs],XtNgridUnits,output->output->graph_units);nargs++;
+       XtSetArg(args[nargs],XtNlabel,output->label);nargs++;
+       sprintf(str,output->output->graph_units,output->output->graph_scale);
+       XtSetArg(args[nargs],XtNgridUnits,str);nargs++;
        XtSetArg(args[nargs],XtNallowResize,True);nargs++;
        XtSetArg(args[nargs],XtNshowGrip,False);nargs++;
 
@@ -411,11 +474,10 @@ void AddRemoveOutput(Output output)
       {
        if(((string=GetProcMeterRC2(module->module->name,output->output->name,"text-font")) ||
            (string=GetProcMeterRC(module->module->name,"text-font")) ||
-           (string=GetProcMeterRC("resources","text-font"))) &&
-          (resource=StringToFont(string)))
-         {XtSetArg(args[nargs],XtNtextFont,resource);nargs++;}
+           (string=GetProcMeterRC("resources","text-font"))))
+         {XtSetArg(args[nargs],XtNtextFont,StringToFont(string));nargs++;}
 
-       XtSetArg(args[nargs],XtNlabel,output->output->name);nargs++;
+       XtSetArg(args[nargs],XtNlabel,output->label);nargs++;
        XtSetArg(args[nargs],XtNallowResize,True);nargs++;
        XtSetArg(args[nargs],XtNskipAdjust,True);nargs++;
        XtSetArg(args[nargs],XtNshowGrip,False);nargs++;
@@ -423,8 +485,34 @@ void AddRemoveOutput(Output output)
        w=XtCreateManagedWidget(output->output->name,procMeterTextWidgetClass,pane,
                                args,nargs);
       }
-    else
-       return;
+    else if(output->type==PROCMETER_BAR)
+      {
+       if(((string=GetProcMeterRC2(module->module->name,output->output->name,"grid-foreground")) ||
+           (string=GetProcMeterRC(module->module->name,"grid-foreground")) ||
+           (string=GetProcMeterRC("resources","grid-foreground"))))
+         {XtSetArg(args[nargs],XtNgridForeground,StringToPixel(string));nargs++;}
+
+       if(((string=GetProcMeterRC2(module->module->name,output->output->name,"grid-min")) ||
+           (string=GetProcMeterRC(module->module->name,"grid-min")) ||
+           (string=GetProcMeterRC("resources","grid-min"))))
+         {XtSetArg(args[nargs],XtNgridMin,StringToInt(string));nargs++;}
+
+       if(((string=GetProcMeterRC2(module->module->name,output->output->name,"grid-max")) ||
+           (string=GetProcMeterRC(module->module->name,"grid-max")) ||
+           (string=GetProcMeterRC("resources","grid-max"))))
+         {XtSetArg(args[nargs],XtNgridMax,StringToInt(string));nargs++;}
+
+       XtSetArg(args[nargs],XtNmin,MINHEIGHT);nargs++;
+
+       XtSetArg(args[nargs],XtNlabel,output->label);nargs++;
+       sprintf(str,output->output->graph_units,output->output->graph_scale);
+       XtSetArg(args[nargs],XtNgridUnits,str);nargs++;
+       XtSetArg(args[nargs],XtNallowResize,True);nargs++;
+       XtSetArg(args[nargs],XtNshowGrip,False);nargs++;
+
+       w=XtCreateManagedWidget(output->output->name,procMeterBarWidgetClass,pane,
+                               args,nargs);
+      }
 
     AddMenuToOutput(w,module,output);
 
@@ -444,6 +532,81 @@ void AddRemoveOutput(Output output)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Move an output.
+
+  Output output1 The output to be moved.
+
+  Output output2 The one that the output is to be moved above or below.
+
+  int direction The direction to move the output (up=1 or down=2).
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void MoveOutput(Output output1,Output output2,int direction)
+{
+ int i,i1=-1,i2=-1;
+
+ for(i=0;i<ndisplayed;i++)
+   {
+    if(displayed[i]==output1)
+       i1=i;
+    if(displayed[i]==output2)
+       i2=i;
+   }
+
+ if(i1==-1 || i2==-1 || i1==i2 || (direction==2 && i1==(i2+1)) || (direction==1 && i1==(i2-1)))
+    return;
+
+ XawPanedSetRefigureMode(pane,False);
+
+ XtUnmanageChild(output1->output_widget);
+
+ for(i=i2;i<ndisplayed;i++)
+   {
+    if(i==i2)
+      {
+       XtManageChild(output1->output_widget);
+       if(direction==2)
+          continue;
+      }
+    if(i!=i1)
+      {
+       XtUnmanageChild(displayed[i]->output_widget);
+       XtManageChild(displayed[i]->output_widget);
+      }
+   }
+
+ XawPanedSetRefigureMode(pane,True);
+
+ ResizePane();
+
+ if(direction==1 && i2>i1)
+   {
+    for(i=i1;i<i2;i++)
+       displayed[i]=displayed[i+1];
+    displayed[i2-1]=output1;
+   }
+ else if(direction==1 && i1>i2)
+   {
+    for(i=i1;i>i2;i--)
+       displayed[i]=displayed[i-1];
+    displayed[i2]=output1;
+   }
+ else if(direction==2 && i2>i1)
+   {
+    for(i=i1;i<i2;i++)
+       displayed[i]=displayed[i+1];
+    displayed[i2]=output1;
+   }
+ else /* if(direction==2 && i1>i2) */
+   {
+    for(i=i1;i>i2;i--)
+       displayed[i]=displayed[i-1];
+    displayed[i2+1]=output1;
+   }
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   The function called by the timeout to terminate the sleep.
 
@@ -508,8 +671,10 @@ static void ResizePane(void)
 
     if(displayed[i]->type==PROCMETER_GRAPH)
        ngraphs++;
-    else
+    else if(displayed[i]->type==PROCMETER_TEXT)
        gsize-=min_size;
+    else if(displayed[i]->type==PROCMETER_BAR)
+       ngraphs++;
 
     rsize-=min_size;
 
@@ -528,7 +693,7 @@ static void ResizePane(void)
     size=gsize/ngraphs;
 
     for(i=0;i<ndisplayed;i++)
-       if(displayed[i]->type==PROCMETER_GRAPH)
+       if(displayed[i]->type==PROCMETER_GRAPH || displayed[i]->type==PROCMETER_BAR)
           XtVaSetValues(displayed[i]->output_widget,vertical?XtNheight:XtNwidth,size,NULL);
    }
  else
