@@ -1,13 +1,13 @@
 /***************************************
-  $Header: /home/amb/procmeter3/RCS/xwindow.h 1.1 1998/09/19 15:22:23 amb Exp $
+  $Header: /home/amb/procmeter3/RCS/xwindow.h 1.3 1999/09/29 19:00:10 amb Exp $
 
-  ProcMeter - A system monitoring program for Linux.
+  ProcMeter - A system monitoring program for Linux - Version 3.2.
 
   Global X windows header file.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1998 Andrew M. Bishop
+  This file Copyright 1998,99 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -33,16 +33,17 @@ void AddMenuToOutput(Widget widget,Module module,Output output);
 /* In xresources.c */
 
 XFontStruct *StringToFont(char *string);
-Pixel *StringToPixel(char *string);
-Boolean *StringToBoolean(char *string);
-int *StringToInt(char *string);
-int *StringToLabelPosition(char *string);
+Pixel StringToPixel(char *string);
+Boolean StringToBoolean(char *string);
+int StringToInt(char *string);
+int StringToLabelPosition(char *string);
 
 /* In xbitmap.c */
 
 extern Pixmap CircleBitmap;
 extern Pixmap TextBitmap;
 extern Pixmap GraphBitmap;
+extern Pixmap BarBitmap;
 
 void CreateBitmaps(Widget w);
 
